@@ -6,8 +6,7 @@ import time
 import xml.etree.ElementTree as ET
 import urllib.parse
 from collections import namedtuple
-from pathlib import Path
-from typing import Dict, Iterable, MutableMapping, NewType, Optional, Set, Tuple, Union
+from typing import Dict, Iterable, MutableMapping, Set, Union
 
 from dateutil.parser import parse as parsedate
 from docutils import nodes, utils
@@ -23,7 +22,6 @@ from .parsing import normalise, ParseException
 
 Entry = namedtuple('Entry', ['kind', 'file'])
 
-Url = NewType("Url", str)
 
 def report_info(env, msg, docname=None, lineno=None):
     '''Convenience function for logging an informational
@@ -216,6 +214,8 @@ def parse_tag_file(doc: ET.ElementTree) -> Dict[str, Union[Entry, FunctionList]]
             member_kind = member.get('kind')
             arglist_text = member.findtext('./arglist')  # If it has an <arglist> then we assume it's a function. Empty <arglist> returns '', not None. Things like typedefs and enums can have empty arglists
 
+            if member_kind == "friend": # ignore friend class definitions because it results in double class entries that will throw a RuntimeError (see below at the end of this function)
+                continue
             if arglist_text and member_kind not in {'variable', 'typedef', 'enumeration', "enumvalue"}:
                 function_list.append((member_symbol, arglist_text, member_kind, join(anchorfile, '#', member.findtext('anchor'))))
             else:
@@ -240,8 +240,8 @@ def parse_tag_file(doc: ET.ElementTree) -> Dict[str, Union[Entry, FunctionList]]
 
 def match_piecewise(candidates: Iterable[str], symbol: str, sep: str='::') -> set:
     """
-    Match the requested symbol reverse piecewise (split on ``::``) against the candidates.
-    This allows you to under-specify the base namespace so that ``"MyClass"`` can match ``my_namespace::MyClass``
+    Match the requested symbol against the candidates.
+    It is allowed to under-specify the base namespace so that ``"MyClass"`` can match ``my_namespace::MyClass``
 
     Args:
         candidates: set of possible matches for symbol
@@ -251,19 +251,10 @@ def match_piecewise(candidates: Iterable[str], symbol: str, sep: str='::') -> se
     Returns:
         set of matches
     """
+    min_length = len(symbol)
     piecewise_list = set()
     for item in candidates:
-        split_symbol = symbol.split(sep)
-        split_item = item.split(sep)
-
-        split_symbol.reverse()
-        split_item.reverse()
-
-        min_length = len(split_symbol)
-
-        split_item = split_item[:min_length]
-
-        if split_symbol == split_item:
+        if symbol == item[-min_length:] and item[-min_length-len(sep):-min_length] in [sep, '']:
             piecewise_list.add(item)
 
     return piecewise_list
@@ -369,17 +360,17 @@ def create_role(app, tag_filename, rootdir, cache_name, pdf=""):
     return find_doxygen_link
 
 
-def extract_configuration(values: tuple) -> Tuple[Union[Url, Path], Optional[Union[Url, Path]], Optional[Union[Url, Path]]]:
+def extract_configuration(values):
     if len(values) == 3:
         tag_filename, rootdir, pdf_filename = values
     elif len(values) == 2:
         tag_filename = values[0]
         if values[1].endswith('.pdf'):
             pdf_filename = values[1]
-            rootdir = None
+            rootdir = ""
         else:
             rootdir = values[1]
-            pdf_filename = None
+            pdf_filename = ""
     else:
         raise ValueError("Config variable `doxylink` is incorrectly configured. Expected a tuple with 2 to 3 "
                             "elements; got %s" % values)
@@ -422,7 +413,7 @@ def fetch_file(app, source, output_path):
                                     "config variable; got %r" % source))
 
 
-def process_configuration(app, tag_filename: Union[Url, Path], rootdir: Optional[Union[Url, Path]], pdf_filename: Optional[Union[Url, Path]]):
+def process_configuration(app, tag_filename, rootdir, pdf_filename):
     """Processes the configured values for ``doxylink`` and ``doxylink_pdf_files`` and warns about potential issues.
 
     The type of builder decides which values shall be used.
